<?php

namespace FM\Service;

use FM\Factory\AppOfflineFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use FM\Service\MinhasTarefasService;
use FM\Service\FuncionariosHierarqService;
use Doctrine\ORM\EntityManager;

class AppOfflineService
{
    protected $connection;
    protected $AppOfflineFactory;
    protected $FuncionariosHierarqService;
    protected $Environment;
    protected $newCode;

    public function __construct(EntityManager $entityManager, AppOfflineFactory $AppOfflineFactory, MinhasTarefasService $MinhasTarefasService, FuncionariosHierarqService $FuncionariosHierarqService, Environment $Environment, NewCode $newCode)
    {
        $this->connection                   = $entityManager->getConnection();
        $this->AppOfflineFactory            = $AppOfflineFactory;
        $this->MinhasTarefasService         = $MinhasTarefasService;
        $this->FuncionariosHierarqService   = $FuncionariosHierarqService;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
    }

    public function getDadosTarefaOffline()
    {
        $parametrizacoes = $this->AppOfflineFactory->getParamOffline();
        $filtro = '';
       
        if ($parametrizacoes[0]['IDTIPOCARREGAOFFLINE'] == "1") {
            $dataInicial = date('d/m/Y');
            $dataFinal = date('d/m/Y', strtotime('+'. $parametrizacoes[0]['QTDIASCARREGAOFFLINE'] + 1 .'days'));
            $filtro = " AND CA.DTHORAAGEND BETWEEN '" . $dataInicial . "' AND '" . $dataFinal . "'";
        } elseif ($parametrizacoes[0]['IDTIPOCARREGAOFFLINE'] == "2") {
            if ($parametrizacoes[0]['DTFINAL'] != null) {
                $dataInicial = $parametrizacoes[0]['DTINICAL'];
                $dataFinal = $parametrizacoes[0]['DTFINAL'];
                $filtro = " AND CA.DTHORAAGEND BETWEEN '" . $dataInicial . "' AND '" . $dataFinal . "'";
            } else {
                $dataInicial = $parametrizacoes[0]['DTINICAL'];
                $filtro = " AND CA.DTHORAAGEND >= '" . $dataInicial . "'";
            }
        } else {
            $dataInicial = date('d/m/Y');
            $dataFinal = date('d/m/Y', strtotime('+ 7 days'));
            $filtro = " AND CA.DTHORAAGEND BETWEEN '" . $dataInicial . "' AND '" . $dataFinal . "'";
        }
        
        $operadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($this->Environment->getCdOperador());
        return  $this->AppOfflineFactory->getDadosTarefaOffline($operadores, $filtro);
    }

    public function getFotosTarefasOffline()
    {
        return $this->AppOfflineFactory->getFotosTarefasOffline();
    }

    public function getItensTarefasOffline()
    {
        return $this->AppOfflineFactory->getItensTarefasOffline();
    }

    public function getParamOffline()
    {
        $cdOperador = $this->Environment->getCdOperador();
        $operadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($this->Environment->getCdOperador());
        $dados = $this->AppOfflineFactory->getParamOffline();
        array_push($dados, $operadores);
        return $dados;
    }

    // Sincronizar tela MINHAS TAREFAS
    public function syncDadosBackend($params)
    {
        if ($params["TAREFAINICIADA"]) {
            $this->syncTarefasIniciadas($params["TAREFAINICIADA"]);
        }
        if ($params["TAREFASFINALIZADAS"]) {
            $this->syncTarefasFinalizadas($params["TAREFASFINALIZADAS"]);
        }
        if ($params["FOTOSANEXADAS"]) {
            $this->syncFotosAnexadas($params["FOTOSANEXADAS"]);
        }
        if ($params["FOTOSREMOVIDAS"]) {
            $this->syncFotosRemovidas($params["FOTOSREMOVIDAS"]);
        }
        if ($params["TAREFASREPORTADAS"]) {
            $this->syncTarefasReportadas($params["TAREFASREPORTADAS"]);
        }
    }

    public function syncTarefasIniciadas($params)
    {
        foreach ($params as $tarefa) {
             $this->MinhasTarefasService->alterarStatusTarefa($tarefa);
        }
    }

    public function syncTarefasFinalizadas($params)
    {
        foreach ($params as $tarefa) {
            $this->MinhasTarefasService->finalizarTarefa($tarefa);
        }
    }

    public function syncTarefasReportadas($params)
    {
        foreach ($params as $reportado) {
             $this->MinhasTarefasService->alterarStatusTarefa($reportado);
        }
    }

    public function syncFotosAnexadas($params)
    {
        foreach ($params as $anexoFoto) {
             $this->MinhasTarefasService->anexarImagensTarefas($anexoFoto);
        }
    }

    public function syncFotosRemovidas($params)
    {
        foreach ($params as $anexoFoto) {
             $this->MinhasTarefasService->excluirFotoTarefas($anexoFoto);
        }
    }

    public function parametrizacaoOfflineApp($params)
    {
        $this->connection->beginTransaction();
        try {
            $cdOperador = $this->Environment->getCdOperador();
            $nrOrg = $this->Environment->getNrOrg();
            $nrParam = $this->AppOfflineFactory->getNrParametrizacoes();
            
            $param = array (
                "IDHABILITAOFFLINE"    => $params['USAOFFLINE'],
                "DTINICARREGAOFFLINE"  => isset($params["DATARANGE"][0]) ? $params["DATARANGE"][0] : null,
                "DTFIMCARREGAOFFLINE"  => isset($params["DATARANGE"][1]) ? $params["DATARANGE"][1] : null,
                "CDOPERULTATU"         => $cdOperador,
                "NRORGULTATU"          => $nrOrg,
                "QTDIASCARREGAOFFLINE" => ($params["XDIAS"] != "") ? $params["XDIAS"] : null,
                "NRPARAMGERAL"         => $nrParam[0]["NRPARAMGERAL"],
                "IDTIPOCARREGAOFFLINE" => $params["TIPOCARREGAMENTO"],
            );
            
            $this->AppOfflineFactory->atualizarParametrizacao($param);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}
